import tomllib
from typing import List
from appdirs import user_config_dir
from pathlib import Path
import yaml
from dataclasses import dataclass


@dataclass
class Models:
    model_name: str
    max_tokens: int


DEFAULT_CONFIG = {
    'interface': 'huggingface',
    # 'max_tokens': 32760,
    'temperature': 1,
    'target_branch': 'master',
    'assignee_id': 10437754,
}

GROQ_MODELS: List[Models] = [
    Models(
        model_name="llama-3.1-70b-versatile",
        max_tokens=8000
    )
    # Models(
    #     model_name="llama-3.1-8b-instruct",
    #     max_tokens=8000
    # )
]


HUGGING_FACE_MODELS: List[Models] = [
    Models(
        model_name="Qwen/Qwen2.5-72B-Instruct",
        max_tokens=32760
    ),
    # HuggingFaceModel(model_name='Qwen/Qwen2.5-1.5B-Instruct', max_tokens=32760),
    # HuggingFaceModel(model_name='microsoft/Phi-3.5-mini-instruct', max_tokens=32760),
    # HuggingFaceModel(model_name='meta-llama/Meta-Llama-3-8B-Instruct', max_tokens=32760),
]


class ConfigManager:
    def __init__(
        self,
        app_name: str,
        app_author: str = None,
        config_filename: str = "config.yaml"
    ):
        self.config_dir = Path(user_config_dir(app_name, app_author))
        self.config_dir.mkdir(parents=True, exist_ok=True)

        # eg: /home/username/.config/gai/config.yaml
        self.config_path = self.config_dir / config_filename
        self.config = self.load_config()

    def load_config(self):
        if not self.config_path.exists():
            self.create_default_config()

        with self.config_path.open('r') as f:
            return yaml.safe_load(f)

    def create_default_config(self):
        default_config = DEFAULT_CONFIG.copy()

        with self.config_path.open('w') as f:
            yaml.dump(default_config, f)
        print(f"Created default config at {self.config_path}")

    def save_config(self):
        with self.config_path.open('w') as f:
            yaml.dump(self.config, f)
        print(f"Saved config to {self.config_path}")

    def update_config(self, key, value):
        self.config[key] = value
        self.save_config()

    def get_config(self, key, default=None):
        return self.config.get(key, default)


def get_app_name():
    try:
        with open('pyproject.toml', 'rb') as f:
            pyproject = tomllib.load(f)
        return pyproject['project']['name']
    except (FileNotFoundError, KeyError):
        return "gai-tool"


if __name__ == "__main__":
    config_manager = ConfigManager(get_app_name())
    target_branch = config_manager.get_config('target_branch')
    print(f"Target branch: {target_branch}")



import ast
from enum import Enum
from typing import Dict, List, Callable
from colorama import Fore, Style
from pick import pick

from gai.src.prompts import Prompts
from gai.src.utils import create_user_message, create_system_message


class OPTIONS(Enum):
    START = "start"
    TRY_AGAIN = "> Try again"
    EXIT = "> Exit"


# TODO: rename this class to avoid cammel case
class DisplayChoices:
    def __init__(self):
        self.history: List[Dict[str, str]] = []

    def parse_response(self, response: str) -> list:
        try:
            result = ast.literal_eval(response)
            if not isinstance(result, list):
                raise ValueError("Response must evaluate to a list")
            return result
        except (ValueError, SyntaxError) as e:
            print(f"Debug - Response that failed parsing: {repr(response)}")  # Show exact string content
            raise ValueError(
                f"\n\nFailed to parse response into list. Error: {str(e)}") from e

    def display_choices(self, items: list, title="Please select an option:"):
        items_refined = items + [OPTIONS.TRY_AGAIN.value, OPTIONS.EXIT.value]

        option, _ = pick(items_refined,
                         title,
                         indicator='*',
                         multiselect=False,
                         min_selection_count=1)
        return option

    def render_choices_with_try_again(
        self,
        user_msg: str,
        ai_client: Callable[[str, str], str],
        sys_prompt: str
    ) -> str:
        choice = OPTIONS.START

        messages: List[Dict[str, str]] = [
            create_system_message(sys_prompt),
            create_user_message(user_msg)
        ]

        response = ai_client(
            user_message=messages.copy(),
        )

        choice = self.run(response)

        while choice == OPTIONS.TRY_AGAIN.value:
            try_again_prompt = Prompts().build_try_again_prompt()
            messages.append(create_system_message(response))
            messages.append(create_user_message(try_again_prompt))

            response = ai_client(
                user_message=messages.copy(),
            )

            choice = self.run(response)

        if choice == OPTIONS.EXIT.value:
            raise Exception("User exited")

        return choice

    def run(self, items: list) -> str:
        selected_item = None
        choices = self.parse_response(items)

        selected_item = self.display_choices(
            items=choices
            # title="Choose an option:"
        )

        print(f"\n{Fore.CYAN}You selected: {selected_item}{Style.RESET_ALL}")
        return selected_item



import os
from typing import Dict, List
from colorama import Fore, Style
from importlib.metadata import version, PackageNotFoundError

import subprocess


def attr_is_defined(args, attr: str) -> bool:
    """
    Check if the specified attribute is defined in the given object.
    """
    return hasattr(args, attr) and getattr(args, attr) is not None


def get_attr_or_default(args, attr: str, default) -> any:
    """
    Get the value of the specified attribute from the object, or return a default value.
    """
    value = getattr(args, attr, default)
    return value if value is not None else default


def get_current_branch() -> str:
    """
    Retrieve the name of the current Git branch.
    """
    result = subprocess.run(
        ["git", "rev-parse", "--abbrev-ref", "HEAD"],
        capture_output=True,
        text=True
    )
    return result.stdout.strip()


def push_changes(remote_repo: str) -> None:
    """
    Push changes to the specified remote repository.
    """
    subprocess.run(["git", "push", remote_repo])


def get_package_version(package_name: str) -> str:
    """
    Get the version of the specified package.
    """
    try:
        return version(package_name)
    except PackageNotFoundError:
        return "Package not found"


def print_tokens(
    user_message: int,
    max_tokens: int
) -> int:
    """
    Print the number of tokens in the system prompt and user message.
    """
    print("\n" + "="*40)
    print(f"{Fore.CYAN}User tokens: {user_message}{Style.RESET_ALL}")
    print(f"{Fore.BLUE}Max tokens: {max_tokens}")
    print(f"={Style.RESET_ALL}"*40 + "\n")


def create_user_message(user_message: str) -> Dict[str, str]:
    """
    Create a user message from the given string.
    """
    return {"role": "user", "content": user_message}


def create_system_message(system_message: str) -> Dict[str, str]:
    """
    Create a system message from the given string.
    """
    return {"role": "system", "content": system_message}


def validate_messages(messages: List[Dict[str, str]]) -> bool:
    """Validate message format."""
    try:
        for message in messages:
            if not isinstance(message, dict):
                raise ValueError("Each message must be a dictionary")
            if "role" not in message and "name" not in message:
                raise ValueError("Each message must have either 'role' or 'name'")
            if "content" not in message:
                raise ValueError("Each message must have 'content'")
        return True
    except Exception as e:
        raise ValueError(f"Message validation failed: {str(e)}")


def get_api_huggingface_key() -> str:
    api_key = os.environ.get("HUGGING_FACE_TOKEN")
    if api_key is None:
        raise ValueError(
            "HUGGING_FACE_TOKEN is not set, please set it in your environment variables")
    return api_key
